//---------------------------------------------------------------------------------
/// \file	CreateSimpleNode.cpp
/// \author	Rob Bateman [robthebloke@hotmail.com]
/// \date	2-7-2004
/// \brief
///	
//---------------------------------------------------------------------------------

#ifdef WIN32
	#define NT_PLUGIN
#endif

// include any maya headers we need.
#include "CreateSimpleNode.h"
#include <maya/MSelectionList.h>
#include <maya/MArgList.h>
#include <maya/MArgDatabase.h>

/// the long and short help flags
const char* g_HelpFlag = "-h";
const char* g_HelpFlagLong = "-help";

/// the text printed when the -h/-help flag is specified
const char* g_HelpText = "This will create a simple node connected to the time attribute";


//---------------------------------------------------------------------------------
/// \brief	This function must create a new syntax description for the mel command.
/// 		This allows us to simplify the parsing of arguments passed to the function
/// 		call.
/// \note	If in mel you type...
///
/// 			help myFunction
///
///			... it should output the following to the script editor
///
/// 			Synopsis: myFunction [flags]
/// 			Flags:
/// 			   -h -help
/// 			  -sl -selected  on|off
///
/// 		Maya uses the MSyntax object we create to produce this information.
///
MSyntax CreateSimpleNode::newSyntax()
{
	MSyntax syn;

	/// add a simple -h flag
	syn.addFlag( g_HelpFlag, g_HelpFlagLong );

	return syn;
}

//---------------------------------------------------------------------------------
///
bool CreateSimpleNode::isUndoable() const {
	return true;
}

//---------------------------------------------------------------------------------
/// \brief	this function is called by maya to return a new instance of our mel function
/// \return	the new mel command object
///
void* CreateSimpleNode::creator() {
	return new CreateSimpleNode;
}


//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus CreateSimpleNode::redoIt()
{
	// the return status
	MStatus stat = MS::kSuccess;

	MSelectionList previous_list;
	stat = MGlobal::getActiveSelectionList(previous_list);

	// attach a function set to this node
	MFnDependencyNode fn;

	// create a simple node
	fn.create( SimpleNode::m_TypeId );

	// store the name
	m_Name = fn.name();

	// set this string as the result from the function
	setResult(m_Name);

	// select the time node
	stat = MGlobal::selectByName( "time1", MGlobal::kReplaceList );

	if(MS::kSuccess != stat) {
		MGlobal::displayError("Could not select");
		return stat;
	}

	// get the selection list containing the time node
	MSelectionList sl;
	stat = MGlobal::getActiveSelectionList(sl);
	if(MS::kSuccess != stat) {
		MGlobal::displayError("Could not get select list");
		return stat;
	}

	// retrieve the MObject for the time1 node
	MObject oTime;
	stat = sl.getDependNode(0,oTime);
	if(MS::kSuccess != stat) {
		MGlobal::displayError("Could not get depend node from selection list");
		return stat;
	}

	// attach a function set to time1
	MFnDependencyNode fnTime( oTime );

	// get a plug to it's outTime attribute
	MPlug outTime = fnTime.findPlug("outTime",&stat);
	if(stat != MS::kSuccess) {
		MGlobal::displayError("Could not get time1.outTime attr");
		return stat;
	}

	// get a plug to the simpleNode.time attribute
	MPlug inTime = fn.findPlug("time");
	if(stat != MS::kSuccess) {
		MGlobal::displayError("Could not get node.time attr");
		return stat;
	}


	// use a dag modifier to connect the two attributes
	MDagModifier m_Mod;
	m_Mod.connect( outTime, inTime );

	// perform the changes to the DAG
	stat = m_Mod.doIt();

	// restore the previous selection list
	stat = MGlobal::setActiveSelectionList(previous_list);

	return stat;
}
//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus CreateSimpleNode::undoIt()
{
	MSelectionList sl,previous;
	MStatus stat;

	// store the previous selection list so that we can restore it later
	stat = MGlobal::getActiveSelectionList(previous);
	if(MS::kSuccess != stat)
	{
		MGlobal::displayError("Could not get select list");
		return stat;
	}

	// select the simple node
	stat = MGlobal::selectByName( m_Name, MGlobal::kReplaceList );
	if(MS::kSuccess != stat)
	{
		MGlobal::displayError("Could not select");
		return stat;
	}

	// then simply delete it
	MGlobal::executeCommand("delete");

	// restore the previous selection list
	stat = MGlobal::setActiveSelectionList(previous);
	return stat;
}

//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus CreateSimpleNode::doIt( const MArgList& args )
{
	// the return status
	MStatus stat = MS::kSuccess;

	// create a parser for the arguments.
	MArgDatabase argData(syntax(),args,&stat);

	// make sure the arguments were parsed OK
	if(stat != MS::kSuccess)
	{
		stat.perror("arguments not specified correctly");
		return stat;
	}

	// if the help flag has been set, simply print the results to the script editor
	if(argData.isFlagSet(g_HelpFlag))
	{
		setResult(g_HelpText);
		return MS::kSuccess;
	}


	return redoIt();
}

