//-------------------------------------------------------------------------------------------------
/// \file	SimpleNode.cpp
/// \date	6-11-04
/// \author	Rob Bateman
/// \brief
///
//-------------------------------------------------------------------------------------------------

#include "SimpleNode.h"
#include <math.h>

const MTypeId SimpleNode::m_TypeId( 0x70032 );
const MString SimpleNode::m_TypeName( "simpleNode" );


/// an connection made to the input time value
MObject SimpleNode::m_aTime;

/// A single output floating point value
MObject SimpleNode::m_aOutput;


//-------------------------------------------------------------------------------------------------
//
void* SimpleNode::creator()
{
	// return a new instance of the node for maya to use
    return new SimpleNode();
}

//-------------------------------------------------------------------------------------------------
//
MStatus SimpleNode::initialize()
{
	// the input attibute is a time attribute that will be connected to the global time node attribute,
	// time1.outTime
	//
	//	Any attribute types that are measured in changeable units, ie distance(cm, mm, miles, feet),
	//	angles( degrees, radians ), time( secs, mins, frames ) etc should be created using the
	//	MFnUnitAttribute function set.
	//
	MFnUnitAttribute uAttr;

    // create the input time attribute
	m_aTime =  uAttr.create( "time", "t", MFnUnitAttribute::kTime );

	// since the input attribute will be connected to the time value, there is little point
	// in storing this attribute within the maya binary file. There is also no point in
	// making the attribute keyable.
	uAttr.setStorable(false);
	uAttr.setKeyable(false);


	//	The MFnNumericAttribute function set allows us to add numeric types such as
	//
	//	bool, int, float, double, point, colour etc.
	//
    MFnNumericAttribute nAttr;

    // create the output attribute
    m_aOutput = nAttr.create( "output", "o",
						   MFnNumericData::kFloat,
						   0.0f);

	// make this attribute a read only output value
	nAttr.setReadable(true);
    nAttr.setKeyable(false);
	nAttr.setWritable(false);


	// add the attributes to the node
	addAttribute(m_aTime);
	addAttribute(m_aOutput);

	// set the attribute dependency. the output attribute will now be evaluated whenever
	// the time input changes.
	//
	attributeAffects(m_aTime,  m_aOutput);

	// done
    return MS::kSuccess;
}


//-------------------------------------------------------------------------------------------------
//
//
MStatus SimpleNode::compute (const MPlug& plug, MDataBlock& data)
{
	// Check that the requested recompute is one of the output values
	//
	if( plug != m_aOutput )
		return MS::kUnknownParameter;

	// get the input time handle
	MDataHandle timeData = data.inputValue(m_aTime);

	// get the output handle
	MDataHandle outputData = data.outputValue(m_aOutput);

	// get the input value as a time value
	MTime Time = timeData.asTime();

	// set the output to the sine of the time (in seconds)
	outputData.set( (float)sin( Time.as(MTime::kSeconds) ) );

	// clean the output plug, ie unset it from dirty so that maya does not re-evaluate it
	data.setClean( plug );

	return MS::kSuccess;
}
